#!/usr/bin/python

import string
import gobject
import avahi

import dbus
from dbus.mainloop.glib import DBusGMainLoop

## setup some globals 
serviceName = "WFUT Server"
serviceType = "_wfut._tcp"
servicePort = 0;
serviceTXT = ""

domain = "local"
host = ""

group = None
rename_count = 12

## Root of http server
serverRoot="./"

def add_service():
    global group, serviceName, serviceType, servicePort, serviceTXT, domain, host
    if group is None:
        group = dbus.Interface(
                bus.get_object( avahi.DBUS_NAME, server.EntryGroupNew()),
                avahi.DBUS_INTERFACE_ENTRY_GROUP)
        group.connect_to_signal('StateChanged', entry_group_state_changed)

    print "Adding service '%s' of type '%s' ..." % (serviceName, serviceType)

    group.AddService(
            avahi.IF_UNSPEC,    #interface
            avahi.PROTO_UNSPEC, #protocol
            0,                  #flags
            serviceName, serviceType,
            domain, host,
            dbus.UInt16(servicePort),
            avahi.string_array_to_txt_array(serviceTXT))
    group.Commit()

def remove_service():
    global group

    if not group is None:
        group.Reset()

def server_state_changed(state):
    if state == avahi.SERVER_COLLISION:
        print "WARNING: Server name collision"
        remove_service()
    elif state == avahi.SERVER_RUNNING:
        add_service()

def entry_group_state_changed(state, error):
    global serviceName, server, rename_count

    print "state change: %i" % state

    if state == avahi.ENTRY_GROUP_ESTABLISHED:
        print "Service established."
    elif state == avahi.ENTRY_GROUP_COLLISION:

        rename_count = rename_count - 1
        if rename_count > 0:
            name = server.GetAlternativeServiceName(name)
            print "WARNING: Service name collision, changing name to '%s' ..." % name
            remove_service()
            add_service()

        else:
            print "ERROR: No suitable service name found after %i retries, exiting." % n_rename
            main_loop.quit()
    elif state == avahi.ENTRY_GROUP_FAILURE:
        print "Error in group state changed", error
        main_loop.quit()
        return

from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer
class WFUTHandler(BaseHTTPRequestHandler):

  def do_GET(self):

    ## If no path is specified, then stick up a little message
    if self.path == "/":
      self.wfile.write("WFUT Server")
      return

    try:
      ## log file access
      print(string.join(["Accessing:", self.path]))

      ## open file and transmit content
      f = open(serverRoot + "/" + self.path)
      self.send_response(200)
      self.send_header('Content-type', 'application/octet-stream');
      self.end_headers();
      self.wfile.write(f.read());
      f.close();

    except IOError:
      self.send_error(404, 'File Not Found: %s' % self.path)


if __name__ == '__main__':
    DBusGMainLoop( set_as_default=True )

    main_loop = gobject.MainLoop()
    bus = dbus.SystemBus()

    ## TODO; Read config file

    import sys,getopt
    optlist, args = getopt.getopt(sys.argv[1:], "p:")

    for (arg, opt) in optlist:
        ## Extract port number
        if arg == '-p': servicePort = int(opt);

    ## TODO: Bind to particular address?
    ws = HTTPServer(('', servicePort), WFUTHandler)

    ## Generate server url
    url = string.join(["http://", ws.server_name, ":", str(ws.server_port), "/"], "");

    print "Server URL is: " + url

    ## Extract port number
    servicePort = ws.server_port;

    server = dbus.Interface(
            bus.get_object( avahi.DBUS_NAME, avahi.DBUS_PATH_SERVER ),
            avahi.DBUS_INTERFACE_SERVER )

    server.connect_to_signal( "StateChanged", server_state_changed )
    server_state_changed( server.GetState() )

    try:

      context = main_loop.get_context()

      while True:
        # Handle commands here
        ws.handle_request();

        # Allow gobject/dbus/avahi to do something
        context.iteration(True)

        # main_loop.run()

    except KeyboardInterrupt:
      pass

    if not group is None:
      group.Free()




#!/usr/bin/python

def go():
	## I think we need an object here which will poll until told to stop.
	## Register some callbacks to spit out servers

        print "Here"
	import sys
	import time
	import dbus, gobject, avahi
	from dbus.mainloop.glib import DBusGMainLoop

	import string

	serviceType = "_wfut._tcp"
	servicePort = 0;
	serviceTXT = ""

	domain = ""
	host = ""

	## Map of discovered servers
	servers = {}

	def service_add(interface, protocol, name, type, domain, flags):


		print "found server"

	        ## Find out more info
	        obj_path = server.ServiceResolverNew(interface, protocol, name, type, domain, avahi.PROTO_UNSPEC, dbus.UInt32(0))

	        resolver = dbus.Interface(bus.get_object(avahi.DBUS_NAME, obj_path),
	                   avahi.DBUS_INTERFACE_SERVICE_RESOLVER)

	        ## Hook up callbacks to 'Found'
	        resolver.connect_to_signal('Found', found)

	def found(interface, protcol, name, type, domain, host, aprotocol, address, port, txt, flags):

	        ## Construct a server url from address and port number.
	        ## Uses '[' ']' incase of IPv6 addresses.
	        servers[name] = "".join([ "http://", "[" , address, "]", ":", str(port)])
	        ## Look for a "url" txt field to override the one constructed above.
	        txt_arr = avahi.txt_array_to_string_array(txt)
	        for rec in txt_arr:
	                (key, value) = rec.split("=")
	                if key == "url":
	                        servers[name] = value
	        print "Found server: " + servers[name]



	if True:
            print "Here"

	    DBusGMainLoop( set_as_default=True )

	    main_loop = gobject.MainLoop()
	    bus = dbus.SystemBus()

	    ## TODO: Hook up service added / removed

	    global server
	    server = dbus.Interface(
	            bus.get_object( avahi.DBUS_NAME, avahi.DBUS_PATH_SERVER ),
	            avahi.DBUS_INTERFACE_SERVER )

	    ## Find services from any interface and protocol
	    interface =  avahi.IF_UNSPEC
	    protocol = avahi.PROTO_UNSPEC

	    ## Use local domain.
	    domain = "local"

	    ## Call method to create new browser, and get back an object
	    ## path for it.
	    obj_path = server.ServiceBrowserNew(interface, protocol, serviceType, domain, dbus.UInt32(0))

	    ## Create browser interface for the new object
	    browser = dbus.Interface(bus.get_object(avahi.DBUS_NAME, obj_path), avahi.DBUS_INTERFACE_SERVICE_BROWSER)

	    ## Hook up signal for new found services
	    browser.connect_to_signal('ItemNew', service_add)

	    try:

		context = main_loop.get_context()
	        ## TODO: loop for short time period
	        while (context.pending()):
			print "tick"
	                time.sleep(0.5)

			# Handle commands here
			context.iteration(True)


		## TODO: Turn off avahi

		## Offer choices to user
		index = 0;
		options = [];
	        for (key, value) in servers.iteritems():
			index = index + 1;
			print(string.join(["Found (", str(index), ") ", key, " -> ", value]))
			options.append(key)


		while True:
			try:
				print "Pick Server: ",
				opt = sys.stdin.readline();
	       			opt = int(opt) - 1;
				break;

			except Exception as e:
				print e

		## get selected server option
		choice = options[opt]
		server = servers[choice];


		return server
		## Fall back to original code with new server choice

	    except KeyboardInterrupt:
	        pass



#!/usr/bin/python
import os, sys
from ConfigParser import ConfigParser
from libwfut import WFUT
from libwfut import WFUTCore

## Read global/local config files
def loadConfig():
	## Construct filename for user config file
	filename = os.path.join(os.environ['HOME'], '.libwfutrc');

	config = ConfigParser();

	## TODO: Move into a setDefaults() method?

	## Set some default values
	config.add_section("general");
	config.set("general", "server_root", "http://white.worldforgedev.org/WFUT/");

	## Read user config file.	
	config.read([filename]);

	## TODO: We do not need to do this all the time...
	## Save config file
	fp = open(filename, "w");
	config.write(fp);
	fp.close();

	return config;

##def printUsage(invoked):
##	print invoked." -u channel_name OPTIONS";
##	print "-u channel name\t (Required) May be '.' to determine from current dir";
##	print "-s system_path\t Path containing system/read-only channel data"
##	print "-S server_root\t Url to remote server root";
##	print "-p local_path\t Path to local updates dir. Current dir is used by default.";

## Load user config
config = loadConfig();

## Setup default values
server_root = config.get("general", "server_root");
channel = ".";
local_path = "./";
system_path = "";
use_avahi = False

## Arguments parsing function
import getopt
optlist, args = getopt.getopt(sys.argv[1:], "u:s:p:vS:hL")

for (arg, opt) in optlist:
	if arg == '-u': channel = opt;
	if arg == '-s': system_path = opt;
	if arg == '-S': server_root = opt;
	if arg == '-p': local_path = opt;	
	if arg == '-v': pass
	if arg == '-h': pass
	if arg == '-L': use_avahi = True

## If scan local network;
if use_avahi:
	try:
		## Use wfut-avahi-client to return a sever root url
		import WFUTAvahiClient;
		server_root = WFUTAvahiClient.go();
	except Exception as e:
		print e;

	

## Check we have required data input
if channel == ".":
	print "No channel specified, will try using current directory";

client = WFUTCore.Client();

client.setChannelName(channel);
client.setLocalPath(local_path);
client.setSystemPath(system_path);
client.setServerRoot(server_root);

client.startUpdates();

while (client.poll()):
	pass

client.__deinit__();



import os
from libwfut import WFUT

#### Select a mirror
##wfut.getMirrorList(server_root + mirror_file, mirrors);
##### TODO: Select mirror
##if mirrors.empty() == False:
##	print mirrors.front().url;

## Set up event handlers
def onDownloadComplete(url, filename, local, updates, tmpfile):
	print "Downloaded", filename;
	fileobject = updates.getFiles()[filename];
	## Log to local list
	local.addFile(fileobject);
	## Record Update
	recordUpdate(fileobject, tmpfile);

def onDownloadFailure(url, filename, reason):
	print "Downloaded Failed", filename, "Reason:", reason;

def onUpdateReason(filename, reason):
	#print filename, reason, WFUT.WFUT_UPDATE_NO_LOCAL;
	pass;

def recordUpdate(fileobject, tmpfile):
	"""Records a file update in the temp file."""
	print(tmpfile)
	try:
		fp = None;
		if os.path.exists(tmpfile):
			## Append to existing file
			fp = open(tmpfile, "at");
		else:
			## Create a new file
			fp = open(tmp_wfut, "wt");
			fp.write("<?xml version=\"1.0\"?>\n");
			fp.write("<fileList dir=\"\">\n");

		fname = WFUT.Encoder_encodeString(fileobject.filename);
		print(fname)
		## TODO: Make sure crc is printed as unsigned.
		entry = "".join(["<file filename=\"", fname, "\" version=\"", str(fileobject.version), "\" crc32=\"",  str(fileobject.crc32), "\" size=\"", str(fileobject.size), "\" execute=\"", str(fileobject.execute), "\"/>\n"]);
		fp.write(entry);

		fp.close();
	except Exception, e:
		print e;

class Client:
	def __init__(self):
		self.mirror_file = "mirrors.xml";
		self.channel_file = "wfut.xml";
		self.tmpfile = "tempwfut.xml";

		self.wfut = WFUT.WFUTClient();
		self.wfut.init();

		self.channel = ".";
		self.local_path = "./";
		self.system_path = "";
		self.server_root = "";

	def setChannelName(self, channel):
		self.channel = channel;

	def setSystemPath(self, system_path):
		self.system_path = system_path;

	def setLocalPath(self, local_path):
		self.local_path = local_path;

	def setServerRoot(self, server_root):
		self.server_root = server_root;

	def startUpdates(self):
		self.mirrors = WFUT.MirrorList();
		self.updates = WFUT.ChannelFileList();
		self.local = WFUT.ChannelFileList();
		self.tmplist = WFUT.ChannelFileList();
		self.system = WFUT.ChannelFileList();
		self.server = WFUT.ChannelFileList();

		local_root = self.local_path + "/" + self.channel + "/";
		local_wfut = self.local_path + "/" + self.channel + "/" + self.channel_file;
		system_wfut = self.system_path + "/" + self.channel + "/" + self.channel_file;
		server_wfut = self.server_root + "/" + self.channel + "/" + self.channel_file;

		self.tmp_wfut = self.local_path + "/" + self.tmpfile;

		## Hook up callbacks
		#  Create a callback binding the updates list and tmp file loations.
		def dc(u,f): return onDownloadComplete(u, f, self.local, self.updates, self.tmp_wfut)
		self.dc = dc;
		self.wfut.DownloadCompleteCB(self.dc);

		self.wfut.DownloadFailedCB(onDownloadFailure);
		self.wfut.UpdateReasonCB(onUpdateReason);

		
		## Read local updates location
		if (os.path.exists(local_wfut)):
			if (self.wfut.getLocalList(local_wfut, self.local) > 0):
				print "Error reading local wfut.xml";
			else:
				if (self.channel == "."):
					 self.channel = self.local.getName();

		## If channel name is still '.', then we cannot proceed.
		if (self.channel == "."):
			raise "Unable to determine local channel name";

		## Look for tmpwfut.xml file and pull in extra updates.
		if (os.path.exists(self.tmp_wfut)):
			if (self.wfut.getLocalList(self.tmp_wfut, self.tmplist) > 0):
				print "Error reading tempwfut.xml";
			else:
				## Copy tmp list into local list
				files = self.tmplist.getFiles();
				for filename, fileobject in files.items():
					self.local.addFile(fileobject)

		## Read system location
		if (os.path.exists(system_wfut)):
			if (self.wfut.getLocalList(system_wfut, self.system) > 0):
				print "Error reading system wfut.xml";
			else:
				pass	

		## Get server list
		if (self.wfut.getFileList(server_wfut, self.server) > 0):
			print "Error reading or downloading server wfut.xml";
		else:
			pass

		## Calculate updates
		self.wfut.calculateUpdates(self.server, self.system, self.local, self.updates, local_root);

		# Set channel name
		self.local.setName(self.server.getName());

		## Download updates
		self.wfut.updateChannel(self.updates, self.server_root + "/" + self.channel, local_root);

	def poll(self):
		return self.wfut.poll();
			
	def finishUpdates(self):	
		## Save download list
		self.wfut.saveLocalList(self.local, local_wfut);
		## Delete temp files.
		os.remove(self.tmp_wfut);

	def __deinit__(self):
		self.wfut.shutdown();

